###################
## Odds matrices ##
###################

OddsMat <- function(alpha,theta1,theta2=theta1,type)
{
	if (type=="N"){return(OddsN(alpha,theta1))}
	if (type=="1L"){return(Odds1L(alpha,theta1))}
	if (type=="ME"){return(OddsME(alpha,theta1,theta2))}
	if (type=="RR"){return(OddsRR(alpha,theta1))}
	if (type=="DD"){return(OddsDD(alpha,theta1))}
	if (type=="RD"){return(OddsRD(alpha,theta1))}
	if (type=="IME"){return(OddsIME(alpha,theta1))}
	if (type=="I"){return(OddsI(alpha,theta1))}
	if (type=="T"){return(OddsT(alpha,theta1))}
	if (type=="Mod"){return(OddsMod(alpha,theta1))}
	if (type=="XOR"){return(OddsXOR(alpha,theta1))}
}

OddsN <- function(a,theta)
{
	return(c(a,a,a,a,a,a,a,a,a))	
}

Odds1L <- function(a,theta)
{
	return(c(a,a,a,a,a,a,a*(1+theta),a*(1+theta),a*(1+theta)))
}

OddsME <- function(a,theta,the2="1")
{
	if (the2=="1")
	{theta2=theta}
	else {theta2=the2}
	return(c(a*(1+theta)^0*(1+theta2)^0,a*(1+theta)^1*(1+theta2)^0,a*(1+theta)^2*(1+theta2)^0,a*(1+theta)^0*(1+theta2)^1,a*(1+theta)^1*(1+theta2)^1,a*(1+theta)^2*(1+theta2)^1,a*(1+theta)^0*(1+theta2)^2,a*(1+theta)^1*(1+theta2)^2,a*(1+theta)^2*(1+theta2)^2))
}

OddsRR <- function(a,theta)
{
	return(c(a,a,a,a,a,a,a,a,a*(1+theta)))
}

OddsDD <- function(a,theta)
{
	return(c(a,a,a,a,a*(1+theta)^1,a*(1+theta)^1,a,a*(1+theta)^1,a*(1+theta)^1))
}

OddsRD <- function(a,theta)
{
	return(c(a,a,a,a,a,a,a,a*(1+theta)^1,a*(1+theta)^1))
}

OddsIME <- function(a,theta)
{
	return(c(a,a,a,a,a*(1+theta)^1,a*(1+theta)^2,a,a*(1+theta)^2,a*(1+theta)^4))
}


OddsI <- function(a,theta)
{
	return(c(a,a,a,a,a*(1+theta),a,a,a,a))
}


OddsT <- function(a,theta)
{
	return(c(a,a,a,a,a,a*(1+theta)^1,a,a*(1+theta)^1,a*(1+theta)^1))
}

OddsMod <- function(a,theta)
{
	return(c(a,a,a,a,a,a*(1+theta)^1,a*(1+theta)^1,a*(1+theta)^1,a*(1+theta)^1))
}

OddsXOR <- function(a,theta)
{
	return(c(a,a,a*(1+theta)^1,a,a,a*(1+theta)^1,a*(1+theta)^1,a*(1+theta)^1,a))
}

###############################
## Theoretical Probabilities ##
###############################

Theo_P_H0_HWE_LD <- function(pA,pB,r)
{
	P0=c(rep(0,9))
	pa=1-pA
	pb=1-pB
	D <- r*sqrt(pA*pB*pa*pb)
	pAB <- D+pA*pB
	paB <- -D+pa*pB
	pAb <- -D+pA*pb
	pab <- D+pa*pb
	if (pAB > 1 | paB > 1 | pAb > 1 | pab > 1 | pAB < 0 | paB < 0 | pAb < 0 | pab < 0)
	{
		print("ERROR R2 impossible!!")
	} 
	else {
		#null hypothesis 
		P0[1]=pAB^2
		P0[2]=2*pAB*pAb
		P0[3]=pAb^2
		P0[4]=2*pAB*paB
		P0[5]=2*pAB*pab+2*pAb*paB
		P0[6]=2*pAb*pab
		P0[7]=paB^2
		P0[8]=2*paB*pab
		P0[9]=pab^2
		return(P0)
	}
}


##########################
## Prevalence and theta ##
##########################


GetPrevalence <- function(alpha,theta,pAB,type)
{
	pDisease <- 0
	odds <- OddsMat(a=alpha,theta1=theta,theta2=theta,type=type)
	pDiseaseCond <- odds/(1+odds)
	for (i in 1:9)
	{
		pDisease <- pDisease+pDiseaseCond[i]*pAB[i]
		
	}
	return(pDisease)
}


GetTheta <- function(theta,alpha,pAB,type,prevalence)
{
	p <- GetPrevalence(alpha=alpha,theta=theta,pAB,type=type)
	return(abs(p-prevalence))
}


OptimTheta <- function(alpha,pAB,type,prevalence)
{
	res <- optimize(f=GetTheta,interval=c(0,100),alpha=alpha,pAB=pAB,type=type,prevalence=prevalence)
	return(res$minimum)

}


#######################
## IndOR
#######################


GetBLog <- function(M)
{
	res <- rep(-1,times=4)
	res[1] <- log(M[5,1])-log(sum(M[c(4,5,6),1]))-log(sum(M[c(2,5,8),1]))-
	log(M[5,2])+log(sum(M[c(4,5,6),2]))+log(sum(M[c(2,5,8),2]))+
	log(M[1,2])-log(sum(M[c(1,2,3),2]))-log(sum(M[c(1,4,7),2]))-
	log(M[1,1])+log(sum(M[c(1,2,3),1]))+log(sum(M[c(1,4,7),1]))
	res[2] <- log(M[6,1])-log(sum(M[c(4,5,6),1]))-log(sum(M[c(3,6,9),1]))-
	log(M[6,2])+log(sum(M[c(4,5,6),2]))+log(sum(M[c(3,6,9),2]))+
	log(M[1,2])-log(sum(M[c(1,2,3),2]))-log(sum(M[c(1,4,7),2]))-
	log(M[1,1])+log(sum(M[c(1,2,3),1]))+log(sum(M[c(1,4,7),1]))
	res[3] <- log(M[8,1])-log(sum(M[c(7,8,9),1]))-log(sum(M[c(2,5,8),1]))-
	log(M[8,2])+log(sum(M[c(7,8,9),2]))+log(sum(M[c(2,5,8),2]))+
	log(M[1,2])-log(sum(M[c(1,2,3),2]))-log(sum(M[c(1,4,7),2]))-
	log(M[1,1])+log(sum(M[c(1,2,3),1]))+log(sum(M[c(1,4,7),1]))
	res[4] <- log(M[9,1])-log(sum(M[c(7,8,9),1]))-log(sum(M[c(3,6,9),1]))-
	log(M[9,2])+log(sum(M[c(7,8,9),2]))+log(sum(M[c(3,6,9),2]))+
	log(M[1,2])-log(sum(M[c(1,2,3),2]))-log(sum(M[c(1,4,7),2]))-
	log(M[1,1])+log(sum(M[c(1,2,3),1]))+log(sum(M[c(1,4,7),1]))
	return(res)
}

CoefDelta1 <- function(M)
{
	# M is 9 vector
	delta <- rep(-1,times=9)
	n <- sum(M)
	delta[1] <- -sqrt((n-M[1])/(n*M[1]))+
		sqrt((M[1]*(n-M[1]))/(n*(M[1]+M[2]+M[3])^2))+
		sqrt((M[1]*(n-M[1]))/(n*(M[1]+M[4]+M[7])^2))
	delta[2] <- sqrt((M[2]*(n-M[2]))/(n*(M[1]+M[2]+M[3])^2))-
		sqrt((M[2]*(n-M[2]))/(n*(M[2]+M[5]+M[8])^2))
	delta[3] <- sqrt((M[3]*(n-M[3]))/(n*(M[1]+M[2]+M[3])^2))
	delta[4] <- -sqrt((M[4]*(n-M[4]))/(n*(M[4]+M[5]+M[6])^2))+
		sqrt((M[4]*(n-M[4]))/(n*(M[1]+M[4]+M[7])^2))
	delta[5] <- sqrt((n-M[5])/(n*M[5]))-
		sqrt((M[5]*(n-M[5]))/(n*(M[4]+M[5]+M[6])^2))-
		sqrt((M[5]*(n-M[5]))/(n*(M[2]+M[5]+M[8])^2))
	delta[6] <- -sqrt((M[6]*(n-M[6]))/(n*(M[4]+M[5]+M[6])^2))
	delta[7] <- sqrt((M[7]*(n-M[7]))/(n*(M[1]+M[4]+M[7])^2))
	delta[8] <- -sqrt((M[8]*(n-M[8]))/(n*(M[2]+M[5]+M[8])^2))
	delta[9] <- 0
	return(delta)
}

CoefDelta2 <- function(M)
{
	# M is 9 vector
	delta <- rep(-1,times=9)
	n <- sum(M)
	delta[1] <- -sqrt((n-M[1])/(n*M[1]))+
		sqrt((M[1]*(n-M[1]))/(n*(M[1]+M[2]+M[3])^2))+
		sqrt((M[1]*(n-M[1]))/(n*(M[1]+M[4]+M[7])^2))
	delta[2] <- sqrt((M[2]*(n-M[2]))/(n*(M[1]+M[2]+M[3])^2))
	delta[3] <- -sqrt((M[3]*(n-M[3]))/(n*(M[3]+M[6]+M[9])^2))+
		sqrt((M[3]*(n-M[3]))/(n*(M[1]+M[2]+M[3])^2))
	delta[4] <- -sqrt((M[4]*(n-M[4]))/(n*(M[4]+M[5]+M[6])^2))+
		sqrt((M[4]*(n-M[4]))/(n*(M[1]+M[4]+M[7])^2))
	delta[5] <- -sqrt((M[5]*(n-M[5]))/(n*(M[4]+M[5]+M[6])^2))
	delta[6] <- sqrt((n-M[6])/(n*M[6]))-
		sqrt((M[6]*(n-M[6]))/(n*(M[4]+M[5]+M[6])^2))-
		sqrt((M[6]*(n-M[6]))/(n*(M[3]+M[6]+M[9])^2))
	delta[7] <- sqrt((M[7]*(n-M[7]))/(n*(M[1]+M[4]+M[7])^2))
	delta[8] <- 0
	delta[9] <- -sqrt((M[9]*(n-M[9]))/(n*(M[3]+M[6]+M[9])^2))
	return(delta)
}

CoefDelta3 <- function(M)
{
	# M is 9 vector
	delta <- rep(-1,times=9)
	n <- sum(M)
	delta[1] <- -sqrt((n-M[1])/(n*M[1]))+
		sqrt((M[1]*(n-M[1]))/(n*(M[1]+M[2]+M[3])^2))+
		sqrt((M[1]*(n-M[1]))/(n*(M[1]+M[4]+M[7])^2))
	delta[2] <- -sqrt((M[2]*(n-M[2]))/(n*(M[2]+M[5]+M[8])^2))+
		sqrt((M[2]*(n-M[2]))/(n*(M[1]+M[2]+M[3])^2))
	delta[3] <- sqrt((M[3]*(n-M[3]))/(n*(M[1]+M[2]+M[3])^2))
	delta[4] <- sqrt((M[4]*(n-M[4]))/(n*(M[1]+M[4]+M[7])^2))
	delta[5] <- -sqrt((M[5]*(n-M[5]))/(n*(M[2]+M[5]+M[8])^2))
	delta[6] <- 0
	delta[7] <- -sqrt((M[7]*(n-M[7]))/(n*(M[7]+M[8]+M[9])^2))+
		sqrt((M[7]*(n-M[7]))/(n*(M[1]+M[4]+M[7])^2))
	delta[8] <- sqrt((n-M[8])/(n*M[8]))-
		sqrt((M[8]*(n-M[8]))/(n*(M[7]+M[8]+M[9])^2))-
		sqrt((M[8]*(n-M[8]))/(n*(M[2]+M[5]+M[8])^2))
	delta[9] <- -sqrt((M[9]*(n-M[9]))/(n*(M[7]+M[8]+M[9])^2))
	return(delta)
}

CoefDelta4 <- function(M)
{
	# M is 9 vector
	delta <- rep(-1,times=9)
	n <- sum(M)
	delta[1] <- -sqrt((n-M[1])/(n*M[1]))+
		sqrt((M[1]*(n-M[1]))/(n*(M[1]+M[2]+M[3])^2))+
		sqrt((M[1]*(n-M[1]))/(n*(M[1]+M[4]+M[7])^2))
	delta[2] <- sqrt((M[2]*(n-M[2]))/(n*(M[1]+M[2]+M[3])^2))
	delta[3] <- -sqrt((M[3]*(n-M[3]))/(n*(M[3]+M[6]+M[9])^2))+
		sqrt((M[3]*(n-M[3]))/(n*(M[1]+M[2]+M[3])^2))
	delta[4] <- sqrt((M[4]*(n-M[4]))/(n*(M[1]+M[4]+M[7])^2))
	delta[5] <- 0
	delta[6] <- -sqrt((M[6]*(n-M[6]))/(n*(M[3]+M[6]+M[9])^2))
	delta[7] <- -sqrt((M[7]*(n-M[7]))/(n*(M[7]+M[8]+M[9])^2))+
		sqrt((M[7]*(n-M[7]))/(n*(M[1]+M[4]+M[7])^2))
	delta[8] <- -sqrt((M[8]*(n-M[8]))/(n*(M[7]+M[8]+M[9])^2))
	delta[9] <- sqrt((n-M[9])/(n*M[9]))-
		sqrt((M[9]*(n-M[9]))/(n*(M[7]+M[8]+M[9])^2))-
		sqrt((M[9]*(n-M[9]))/(n*(M[3]+M[6]+M[9])^2))
	return(delta)
}

VarDelta <- function(M,ind=1)
{
	if (ind==1)
	{
		Co <- CoefDelta1(M)
	}
	if (ind==2)
	{
		Co <- CoefDelta2(M)
	}
	if (ind==3)
	{
		Co <- CoefDelta3(M)
	}
	if (ind==4)
	{
		Co <- CoefDelta4(M)
	}
	n <- sum(M)
	res <- sum(Co^2)
	for (i in 1:8){
		for (j in (i+1):9){
			res <- res-2*Co[i]*Co[j]*sqrt((M[i]/(n-M[i]))*((M[j]/(n-M[j]))))
		}
	}
	return(res)
}

CovDelta <- function(M,ind1=1,ind2=2)
{
	if (ind1==1){Co1 <- CoefDelta1(M)}
	if (ind1==2){Co1 <- CoefDelta2(M)}
	if (ind1==3){Co1 <- CoefDelta3(M)}
	if (ind1==4){Co1 <- CoefDelta4(M)}
	if (ind2==1){Co2 <- CoefDelta1(M)}
	if (ind2==2){Co2 <- CoefDelta2(M)}
	if (ind2==3){Co2 <- CoefDelta3(M)}
	if (ind2==4){Co2 <- CoefDelta4(M)}
	n <- sum(M)
	res <- 0
	for (i in 1:9)
	{
		for (j in 1:9)
		{
			if (i==j){res <- res+Co1[i]*Co2[j]}
			else{res <- res-Co1[i]*Co2[j]*sqrt((M[i]/(n-M[i]))*((M[j]/(n-M[j]))))}
		}
	}
	return(res)
}

IndOR <- function(M) ##### M is a 9x2 table where the first column is made by counts for the nine genotypes in cases and the second column stores counts for controls
{
	B <- GetBLog(M)
	VarCov <- matrix(-1,ncol=4,nrow=4)
	for (i in 1:4)
	{	
		VarCov[i,i] <- VarDelta(M[,1],ind=i)+VarDelta(M[,2],ind=i)
	}
	for (i in 1:3)
	{
		for (j in (i+1):4)
		{
			tmp <- CovDelta(M[,1],ind1=i,ind2=j)+CovDelta(M[,2],ind1=i,ind2=j)
			VarCov[i,j] <- tmp
			VarCov[j,i] <- tmp
		}
	}
	Inv <- try(solve(VarCov))
	if (!inherits(Inv,"try-error")){S <- B%*%solve(VarCov)%*%B;return(list(statistic=S,p.value=1-pchisq(S,df=4)))}
	else (return((-1)))
}

#################
## Power ###
#################



GetPower <- function(pDisease,pDiseaseBase,type,pA,pB,nInd,ylim=c(0,1),lege,r,ratioCC)
{
	alpha <- pDiseaseBase/(1-pDiseaseBase)
	pAB <- Theo_P_H0_HWE_LD(pA=pA,pB=pB,r=r)
	theta <- OptimTheta(alpha=alpha,pAB=pAB,type=type,prevalence=pDisease)
	odds <- OddsMat(alpha=alpha,theta1=theta,theta2=theta,type=type)
	pDiseaseCond <- odds/(1+odds)
	pAB_Cases <- pDiseaseCond*pAB/pDisease
	pAB_Controls <- (1-pDiseaseCond)*pAB/(1-pDisease)
	nCases <- nInd
	nControls <- nCases*ratioCC
	TD <- cbind(nCases*pAB_Cases,nControls*pAB_Controls)
	IndOR <- 1-pchisq(qchisq(0.95,df=4),df=4,ncp=max(0,IndOR(TD)$statistic))
	return(list(alpha=alpha,theta=theta,IndOR=IndOR))
}

