source("./FunctionsIndOR.R")

pA <- 0.6
pB <- 0.7
pDisease <- 0.01

LD <- c(0,0.05,0.15)
ratioCC <- c(1,2,5)

options(digits=2)

nInd <- 2000 ### Number of cases in the cohort

##############
# Type N
##############

type <- "N"
alpha <- 0.01/(1-0.01)
pDiseaseBase <- alpha/(1+alpha)

cat("------------------------\n")
cat("-- Null Model         --\n")
cat("------------------------\n")
for (r in LD)
{
	for (ratio in ratioCC)
	{
		res <- GetPower(pDisease=pDisease,pDiseaseBase=pDiseaseBase,type=type,pA=pA,pB=pB,nInd=nInd,ylim=c(0,1),lege="topleft",r=r,ratioCC=ratio)
		cat("alpha=",res$alpha,"theta=",res$theta,"r=",r,"nCas=",nInd,"nCont=",nInd*ratio,"IndOR:",res$IndOR,"\n")
	}
	cat("-----------\n")
}

##############
# Type 1L
##############

type <- "1L"
alpha <- 0.009
pDiseaseBase <- alpha/(1+alpha)

cat("--------------\n")
cat("-- Model 1L --\n")
cat("--------------\n")
for (r in LD)
{
	for (ratio in ratioCC)
	{
		res <- GetPower(pDisease=pDisease,pDiseaseBase=pDiseaseBase,type=type,pA=pA,pB=pB,nInd=nInd,ylim=c(0,1),lege="topleft",r=r,ratioCC=ratio)
		cat("alpha=",res$alpha,"theta=",res$theta,"r=",r,"nCas=",nInd,"nCont=",nInd*ratio,"IndOR:",res$IndOR,"\n")
	}
	cat("-----------\n")
}


##############
# Type ME
##############

type <- "ME"
alpha <- 0.005
pDiseaseBase <- alpha/(1+alpha)

cat("--------------\n")
cat("-- Model ME --\n")
cat("---------------\n")
cat("-- a        a(1+t)   a(1+t)^2 --\n")
cat("-- a(1+t)   a(1+t)^2 a(1+t)^3 --\n")
cat("-- a(1+t)^2 a(1+t)^3 a(1+t)^4 --\n")
cat("------------\n")
for (r in LD)
{
	for (ratio in ratioCC)
	{
		res <- GetPower(pDisease=pDisease,pDiseaseBase=pDiseaseBase,type=type,pA=pA,pB=pB,nInd=nInd,ylim=c(0,1),lege="topleft",r=r,ratioCC=ratio)
		cat("alpha=",res$alpha,"theta=",res$theta,"r=",r,"nCas=",nInd,"nCont=",nInd*ratio,"IndOR:",res$IndOR,"\n")
	}
	cat("-----------\n")
}


##############
# Type RR
##############

type <- "RR"
alpha <- 0.00995
pDiseaseBase <- alpha/(1+alpha)

cat("--------------\n")
cat("-- Model RR --\n")
cat("--------------\n")
cat("-- a a a --\n")
cat("-- a a a --\n")
cat("-- a a a(1+t) --\n")
cat("------------\n")
for (r in LD)
{
	for (ratio in ratioCC)
	{
		res <- GetPower(pDisease=pDisease,pDiseaseBase=pDiseaseBase,type=type,pA=pA,pB=pB,nInd=nInd,ylim=c(0,1),lege="topleft",r=r,ratioCC=ratio)
		cat("alpha=",res$alpha,"theta=",res$theta,"r=",r,"nCas=",nInd,"nCont=",nInd*ratio,"IndOR:",res$IndOR,"\n")
	}
	cat("-----------\n")
}

##############
# Type DD
##############

type <- "DD"
alpha <- 0.009
pDiseaseBase <- alpha/(1+alpha)
pDiseaseBase

cat("--------------\n")
cat("-- Model DD --\n")
cat("--------------\n")
cat("-- a a      a --\n")
cat("-- a a(1+t) a(1+t) --\n")
cat("-- a a(1+t) a(1+t) --\n")
cat("------------\n")
for (r in LD)
{
	for (ratio in ratioCC)
	{
		res <- GetPower(pDisease=pDisease,pDiseaseBase=pDiseaseBase,type=type,pA=pA,pB=pB,nInd=nInd,ylim=c(0,1),lege="topleft",r=r,ratioCC=ratio)
		cat("alpha=",res$alpha,"theta=",res$theta,"r=",r,"nCas=",nInd,"nCont=",nInd*ratio,"IndOR:",res$IndOR,"\n")
	}
	cat("-----------\n")
}

##############
# Type RD
##############

type <- "RD"
alpha <- 0.0096
pDiseaseBase <- alpha/(1+alpha)
pDiseaseBase

cat("--------------\n")
cat("-- Model RD --\n")
cat("--------------\n")
cat("-- a a      a --\n")
cat("-- a a      a --\n")
cat("-- a a(1+t) a(1+t) --\n")
cat("------------\n")
for (r in LD)
{
	for (ratio in ratioCC)
	{
		res <- GetPower(pDisease=pDisease,pDiseaseBase=pDiseaseBase,type=type,pA=pA,pB=pB,nInd=nInd,ylim=c(0,1),lege="topleft",r=r,ratioCC=ratio)
		cat("alpha=",res$alpha,"theta=",res$theta,"r=",r,"nCas=",nInd,"nCont=",nInd*ratio,"IndOR:",res$IndOR,"\n")
	}
	cat("-----------\n")
}


##############
# Type IME
##############

type <- "IME"
alpha <- 0.0095
pDiseaseBase <- alpha/(1+alpha)
pDiseaseBase

cat("---------------\n")
cat("-- Model IME --\n")
cat("---------------\n")
cat("-- a a        a --\n")
cat("-- a a(1+t)   a(1+t)^2 --\n")
cat("-- a a(1+t)^2 a(1+t)^4 --\n")
cat("------------\n")
for (r in LD)
{
	for (ratio in ratioCC)
	{
		res <- GetPower(pDisease=pDisease,pDiseaseBase=pDiseaseBase,type=type,pA=pA,pB=pB,nInd=nInd,ylim=c(0,1),lege="topleft",r=r,ratioCC=ratio)
		cat("alpha=",res$alpha,"theta=",res$theta,"r=",r,"nCas=",nInd,"nCont=",nInd*ratio,"IndOR:",res$IndOR,"\n")
	}
	cat("-----------\n")
}

##############
# Type T
##############

type <- "T"
alpha <- 0.0096
pDiseaseBase <- alpha/(1+alpha)
pDiseaseBase

cat("-------------\n")
cat("-- Model T --\n")
cat("-------------\n")
cat("-- a a      a --\n")
cat("-- a a      a(1+t) --\n")
cat("-- a a(1+t) a(1+t) --\n")
cat("------------\n")
for (r in LD)
{
	for (ratio in ratioCC)
	{
		res <- GetPower(pDisease=pDisease,pDiseaseBase=pDiseaseBase,type=type,pA=pA,pB=pB,nInd=nInd,ylim=c(0,1),lege="topleft",r=r,ratioCC=ratio)
		cat("alpha=",res$alpha,"theta=",res$theta,"r=",r,"nCas=",nInd,"nCont=",nInd*ratio,"IndOR:",res$IndOR,"\n")
	}
	cat("-----------\n")
}


##############
# Type I
##############

type <- "I"
alpha <- 0.0095
pDiseaseBase <- alpha/(1+alpha)
pDiseaseBase

cat("-------------\n")
cat("-- Model I --\n")
cat("-------------\n")
cat("-- a a      a --\n")
cat("-- a a(1+t) a --\n")
cat("-- a a      a --\n")
cat("------------\n")
for (r in LD)
{
	for (ratio in ratioCC)
	{
		res <- GetPower(pDisease=pDisease,pDiseaseBase=pDiseaseBase,type=type,pA=pA,pB=pB,nInd=nInd,ylim=c(0,1),lege="topleft",r=r,ratioCC=ratio)
		cat("alpha=",res$alpha,"theta=",res$theta,"r=",r,"nCas=",nInd,"nCont=",nInd*ratio,"IndOR:",res$IndOR,"\n")
	}
	cat("-----------\n")
}


##############
# Type 10
##############

type <- "Mod"
alpha <- 0.0088
pDiseaseBase <- alpha/(1+alpha)
pDiseaseBase

cat("---------------\n")
cat("-- Model Mod --\n")
cat("---------------\n")
cat("-- a a      a --\n")
cat("-- a a      a(1+t) --\n")
cat("-- a(1+t) a(1+t) a(1+t) --\n")
cat("------------\n")
for (r in LD)
{
	for (ratio in ratioCC)
	{
		res <- GetPower(pDisease=pDisease,pDiseaseBase=pDiseaseBase,type=type,pA=pA,pB=pB,nInd=nInd,ylim=c(0,1),lege="topleft",r=r,ratioCC=ratio)
		cat("alpha=",res$alpha,"theta=",res$theta,"r=",r,"nCas=",nInd,"nCont=",nInd*ratio,"IndOR:",res$IndOR,"\n")
	}
	cat("-----------\n")
}

##############
# Type 11
##############

type <- "XOR"
alpha <- 0.0095
pDiseaseBase <- alpha/(1+alpha)
pDiseaseBase

cat("---------------\n")
cat("-- Model XOR --\n")
cat("---------------\n")
cat("-- a a      a(1+t) --\n")
cat("-- a a      a(1+t) --\n")
cat("-- a(1+t) a(1+t) a --\n")
cat("------------\n")
for (r in LD)
{
	for (ratio in ratioCC)
	{
		res <- GetPower(pDisease=pDisease,pDiseaseBase=pDiseaseBase,type=type,pA=pA,pB=pB,nInd=nInd,ylim=c(0,1),lege="topleft",r=r,ratioCC=ratio)
		cat("alpha=",res$alpha,"theta=",res$theta,"r=",r,"nCas=",nInd,"nCont=",nInd*ratio,"IndOR:",res$IndOR,"\n")
	}
	cat("-----------\n")
}

